#cs(module ssax-eg mzscheme

     (require (lib "ssax.ss" "ssax")
              (lib "myenv.ss" "ssax")
              (lib "input-parse.ss" "ssax")
              (prefix su: (lib "test.ss" "schemeunit"))
              (lib "text-ui.ss" "schemeunit")
              "sxml-to-html.ss"
              "sxml-tree-transform.ss")
     
     (provide count-paragraphs
              parse-xml
              microsoftize
              strip-tags
              document
              run-tests)
     
     (define count-paragraphs
       (SSAX:make-parser
        NEW-LEVEL-SEED
        (lambda (name attrs namespaces content seed)
          (if (equal? name 'p)
              (+ 1 seed)
              seed))
        FINISH-ELEMENT
        (lambda (name attrs namespaces parent-seed
seed)
          seed)
        CHAR-DATA-HANDLER
        (lambda (string1 string2 seed)
          seed)
        ))

     ;; CONTRACT:
     ;;   flatten : tree -> list
     ;;
     ;; PURPOSE:
     ;;   flatten converts a tree (represented as a list of lists)
     ;;   into a flat list containing on the data elements of the tree
     ;;
     ;; EXAMPLE:
     ;;   (flatten '((1 2 3) 4 (5 (6)))) = (1 2 3 4 5 6)
     (define (flatten tree)
       (cond ((null? tree) '())
             (else
              (cond ((pair? (car tree))
                     (flatten (append (car tree) (cdr tree))))
                    (else (cons (car tree)
                                (flatten (cdr tree))))))))
     
     
     ;; In-memory strip tags using post-order transform
     (define (strip-tags tree)
         (flatten 
          (post-order tree
                      `((*default* . ,(lambda (tree)
                                        (cdr tree)))
                        (*text* . ,(lambda (text)
                                     text))))))
     
     ;; In-memory microsoftize-ation using post-order transform
     (define (microsoftize tree)
       (post-order tree
                   `((h1 . ,(lambda (tree)
                              `(font (@ (size "7"))
                                     ,@(cdr tree))))
                     (h2 . ,(lambda (tree)
                              `(font (@ (size "6"))
                                     ,@(cdr tree))))
                     (*default* . ,(lambda (tree)
                                     tree))
                     (*text* . ,(lambda (tree)
                                  tree)))))
     
     (define document
       "<html>
          <head><title>Hello</title></head>
          <body>    
            <h1>Heading 1</h1>
              <p>A paragraph</p>
            <h2>Heading 2</h2>
              <p>A paragraph with <a href=\"here.htm\">a link</a> inside</p>
            <h2>Heading 2</h2>
              <p>A paragraph</p>
          </body>
        </html>")
 
     
     (define (parse-xml input-port)
       (SSAX:XML->SXML input-port (list)))
     
     (define tests
       (su:make-test-suite
        "SSAX Tests"
        (su:make-test-case
         "Count paragraphs"
         (su:assert =
                    (count-paragraphs 
                     (open-input-string "<body><p>Here</p><li><p>There</p></li></body>") 0)
                    2))
        
        (su:make-test-case
         "Parse XML"
         (su:assert-equal?
          (parse-xml (open-input-string "<html><head><title>Hello</title></head><body>Hi</body></html>"))
          '(*TOP* (html (head (title "Hello")) (body "Hi")))))
        
        (su:make-test-case
         "SXML to HTML"
         (su:assert-equal?
          (SXML->HTML '(html (head (title "Hello")) (body "Hi")))
          "<html><head><title>Hello</title>\n</head>\n<body>Hi</body>\n</html>\n"))
        
        (su:make-test-case
         "Microsoftize in-memory"
         (su:assert-equal?
          '(|*TOP*|
            (html
             (head (title "Hello"))
             (body
              (font (@ (size "7")) "Heading 1")
              (p "A paragraph")
              (font (@ (size "6")) "Heading 2")
              (p "A paragraph with " (a (@ (href "here.htm")) "a link") " inside")
              (font (@ (size "6")) "Heading 2")
              (p "A paragraph"))))
           (microsoftize (parse-xml (open-input-string document)))))
        
        (su:make-test-case
         "Flatten test"
         (su:assert-equal?
          '(1 2 3 4 5 6)
          (flatten '((1 2 3) 4 (5 (6))))))
        
        (su:make-test-case
         "Strip tags in memory"
         (su:assert-equal?
          (strip-tags (parse-xml (open-input-string document)))
          '("Hello"
            "Heading 1"
            "A paragraph"
            "Heading 2"
            "A paragraph with "
            "here.htm"
            "a link"
            " inside"
            "Heading 2"
            "A paragraph")))
          
        ))
     
     (define (run-tests)
       (test/text-ui tests))
         
     )